Option Explicit On

'importing MBT libs, they need to be referenced in Project -> Add reference
Imports MBTCOMLib
Imports MBTQUOTELib
Imports MBTORDERSLib

Public Class Form1

    'implementing the quotes interface
    Implements IMbtQuotesNotify

    'declaring some MBT object variables
    Private WithEvents oComMgr As MbtComMgr
    Private WithEvents oOrders As MbtOrderClient
    Private WithEvents oQuotes As MbtQuotes
    Private oAct As MbtAccount

    Private oPositions As MbtPositions
    Private oPos As MbtPosition

    Private oOrderHists As MbtOrderHistories
    Private oOrderHist As MbtOrderHistory

    Private oOpenOrders As MbtOpenOrders
    Private oOpenOrder As MbtOpenOrder

    'declaring some other variables
    Private bGotInitialLogonSucceed As Boolean
    Private bComMgrIsConnected As Boolean
    Private bOrderClientIsConnected As Boolean
    Private bQuotesIsConnected As Boolean

    Private username As String
    Private password As String
    Private sdkId As Integer

    Private aSyms(10) As String
    Private symCount As Integer
    Private aPair As String

    Private bidGBP, askGBP, bidEUR, askEUR, bidJPY, askJPY, bidCHF, askCHF As Double
    Private mdBid, mdAsk As Double

    Private stopwatch As Stopwatch



    'These subs handle the loading and closeing events of the application
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        bGotInitialLogonSucceed = False
        bComMgrIsConnected = False
        bOrderClientIsConnected = False
        bQuotesIsConnected = False

        stopwatch = New Stopwatch()

        aSyms(0) = "EUR/USD"
        aSyms(1) = "GBP/USD"
        aSyms(2) = "USD/JPY"
        aSyms(3) = "USD/CHF"

        username = ""
        password = ""
        sdkId = 9

        'instantiate the main MBT application class object
        oComMgr = New MbtComMgr
        'retrieving the orderclient, quotes, positions, orderhistories and order-objects
        oOrders = oComMgr.OrderClient
        oQuotes = oComMgr.Quotes
        oPositions = oOrders.Positions
        oOrderHists = oOrders.OrderHistories
        oOpenOrders = oOrders.OpenOrders

        oOrders.SilentMode = True
        oComMgr.EnableSplash(False)
        cmbPair.SelectedIndex = 0
        cmbType.SelectedIndex = 0
        aPair = "EUR/USD"
        PutLog("Application starting - MBT Navigator Version: " & oComMgr.NavVersion)
        Me.Show()
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        oQuotes.UnadviseAll(Me)
        oQuotes = Nothing
        oOrders = Nothing
        oComMgr = Nothing
        stopwatch = Nothing
    End Sub



    'these subs handle some of the MBT events

    'handleing the OnHistoryAdded event, empty in my example
    Private Sub oOrders_OnHistoryAdded(ByVal pHist As MbtOrderHistory) Handles oOrders.OnHistoryAdded

    End Sub

    'handleing the OnPositionUpdated event, empty in my example
    Private Sub oOrders_OnPositionUpdated(ByVal pPos As MbtPosition) Handles oOrders.OnPositionUpdated

    End Sub

    'handleing the OnLogonSucceed-event, and loading the default account
    Private Sub oOrders_OnLogonSucceed() Handles oOrders.OnLogonSucceed
        oAct = oOrders.Accounts.DefaultAccount
        oAct.Load()
        PutLog("Default account loaded")
        PutLog("Account: " & oAct.Account)
        Me.Text = Me.Text & " - " & oComMgr.UserName
    End Sub

    'handleing the OnAccountLoaded-event, advising EUR/USD
    Private Sub oOrders_OnAccountLoaded(ByVal pAct As MBTORDERSLib.MbtAccount) Handles oOrders.OnAccountLoaded
        oQuotes.AdviseSymbol(Me, "EUR/USD", 1)
        PutLog("Advised " & aPair)
    End Sub

    'this event fires after an order is executed
    Private Sub oOrders_OnExecute(ByVal pOrd As MbtOpenOrder) Handles oOrders.OnExecute
        stopwatch.Stop()
        lblOpen.Text = "Executed in " & stopwatch.ElapsedMilliseconds & "ms"
        stopwatch.Reset()
    End Sub

    'tracing of the various servers connection-states
    Private Sub oComMgr_OnHealthUpdate(ByVal index As MBTCOMLib.enumServerIndex, ByVal state As MBTQUOTELib.enumConnectionState) Handles oComMgr.OnHealthUpdate
        Select Case index
            Case enumServerIndex.siPerms
                If state = enumConnectionState.csLoggedIn Then
                    bComMgrIsConnected = True
                    lblPerm.Image = My.Resources.green
                    PutLog("C-STATE: Perms logged in")
                ElseIf state = enumConnectionState.csConnecting Or state = enumConnectionState.csConnected Or state = enumConnectionState.csLoggingIn Then
                    bComMgrIsConnected = False
                    lblPerm.Image = My.Resources.yellow
                    PutLog("C-STATE: Perms connecting")
                ElseIf state = enumConnectionState.csIndeterminate Then
                    bComMgrIsConnected = False
                    lblPerm.Image = My.Resources.pink
                    PutLog("C-STATE: Perms state is indeterminate")
                ElseIf state = enumConnectionState.csDisconnected Then
                    bComMgrIsConnected = False
                    lblPerm.Image = My.Resources.red
                    PutLog("C-STATE: Perms disconnected")
                End If
            Case enumServerIndex.siOrders
                If state = enumConnectionState.csLoggedIn Then
                    bOrderClientIsConnected = True
                    lblOrder.Image = My.Resources.green
                    PutLog("C-STATE: Orders logged in")
                ElseIf state = enumConnectionState.csConnecting Or state = enumConnectionState.csConnected Or state = enumConnectionState.csLoggingIn Then
                    bOrderClientIsConnected = False
                    lblOrder.Image = My.Resources.yellow
                    PutLog("C-STATE: Orders connecting")
                ElseIf state = enumConnectionState.csIndeterminate Then
                    bOrderClientIsConnected = False
                    lblOrder.Image = My.Resources.pink
                    PutLog("C-STATE: Orders state is indeterminate")
                ElseIf state = enumConnectionState.csDisconnected Then
                    bOrderClientIsConnected = False
                    lblOrder.Image = My.Resources.red
                    PutLog("C-STATE: Orders disconnected")
                End If
            Case enumServerIndex.siQuotes
                If state = enumConnectionState.csLoggedIn Then
                    bQuotesIsConnected = True
                    lblQuote.Image = My.Resources.green
                    PutLog("C-STATE: Quotes logged in")
                ElseIf state = enumConnectionState.csConnecting Or state = enumConnectionState.csConnected Or state = enumConnectionState.csLoggingIn Then
                    bQuotesIsConnected = False
                    lblQuote.Image = My.Resources.yellow
                    PutLog("C-STATE: Quotes connecting")
                ElseIf state = enumConnectionState.csIndeterminate Then
                    bQuotesIsConnected = False
                    lblQuote.Image = My.Resources.pink
                    PutLog("C-STATE: Quotes state is indeterminate")
                ElseIf state = enumConnectionState.csDisconnected Then
                    bQuotesIsConnected = False
                    lblQuote.Image = My.Resources.red
                    PutLog("C-STATE: Quotes disconnected")
                End If
        End Select
    End Sub




    'Recieving realtime quotes thru the mbtQuotes interface

    'Implementation of OnQuoteData of the mbtQuotes interface
    Private Sub OnQuoteData(ByRef pQuote As QUOTERECORD) Implements IMbtQuotesNotify.OnQuoteData
        Dim lastprice As Double
        Dim lastvol As Long
        Dim lvolume As Long

        lastprice = pQuote.dLast
        lastvol = pQuote.lLastSize
        lvolume = pQuote.lVolume
        Times.rtbTimes.Text = lastprice & "  " & lastvol & "  " & lvolume & vbNewLine & Times.rtbTimes.Text

        mdBid = pQuote.dBid
        mdAsk = pQuote.dAsk

        'this is just to split up the quote for displaying pips and subpips
        If aPair = "GBP/USD" Or aPair = "EUR/USD" Or aPair = "USD/CHF" Or aPair = "USD/JPY" Then
            Select Case Len(CStr(mdBid))
                Case 1
                    lblBidPre.Text = mdBid & ".00"
                    cmdBid.Text = "00"
                    lblBidSub.Text = "00"
                Case 3
                    lblBidPre.Text = mdBid & "0"
                    cmdBid.Text = "00"
                    lblBidSub.Text = "00"
                Case 4
                    lblBidPre.Text = mdBid
                    cmdBid.Text = "00"
                    lblBidSub.Text = "00"
                Case 5
                    lblBidPre.Text = Microsoft.VisualBasic.Left(mdBid, 4)
                    cmdBid.Text = Microsoft.VisualBasic.Right(mdBid, 1) & "0"
                    lblBidSub.Text = "00"
                Case 6
                    lblBidPre.Text = Microsoft.VisualBasic.Left(mdBid, 4)
                    cmdBid.Text = Microsoft.VisualBasic.Right(mdBid, 2)
                    lblBidSub.Text = "00"
                Case 7
                    lblBidPre.Text = Microsoft.VisualBasic.Left(mdBid, 4)
                    cmdBid.Text = Microsoft.VisualBasic.Mid(mdBid, 5, 2)
                    lblBidSub.Text = Microsoft.VisualBasic.Right(mdBid, 1) & 0
                Case 8
                    lblBidPre.Text = Microsoft.VisualBasic.Left(mdBid, 4)
                    cmdBid.Text = Microsoft.VisualBasic.Mid(mdBid, 5, 2)
                    lblBidSub.Text = Microsoft.VisualBasic.Right(mdBid, 2)
            End Select
            Select Case Len(CStr(mdAsk))
                Case 1
                    lblAskPre.Text = mdAsk & ".00"
                    cmdAsk.Text = "00"
                    lblAskSub.Text = "00"
                Case 3
                    lblAskPre.Text = mdAsk & "0"
                    cmdAsk.Text = "00"
                    lblAskSub.Text = "00"
                Case 4
                    lblAskPre.Text = mdAsk
                    cmdAsk.Text = "00"
                    lblAskSub.Text = "00"
                Case 5
                    lblAskPre.Text = Microsoft.VisualBasic.Left(mdAsk, 4)
                    cmdAsk.Text = Microsoft.VisualBasic.Right(mdAsk, 1) & "0"
                    lblAskSub.Text = "00"
                Case 6
                    lblAskPre.Text = Microsoft.VisualBasic.Left(mdAsk, 4)
                    cmdAsk.Text = Microsoft.VisualBasic.Right(mdAsk, 2)
                    lblAskSub.Text = "00"
                Case 7
                    lblAskPre.Text = Microsoft.VisualBasic.Left(mdAsk, 4)
                    cmdAsk.Text = Microsoft.VisualBasic.Mid(mdAsk, 5, 2)
                    lblAskSub.Text = Microsoft.VisualBasic.Right(mdAsk, 1) & 0
                Case 8
                    lblAskPre.Text = Microsoft.VisualBasic.Left(mdAsk, 4)
                    cmdAsk.Text = Microsoft.VisualBasic.Mid(mdAsk, 5, 2)
                    lblAskSub.Text = Microsoft.VisualBasic.Right(mdAsk, 2)
            End Select
        End If

    End Sub

    'Implementation of OnLevel2Data of the mbtQuotes interface - this NEEDS to be defined, even if you dont need level2 data
    Private Sub OnLevel2Data(ByRef pRec As LEVEL2RECORD) Implements IMbtQuotesNotify.OnLevel2Data
    End Sub

    'Implementation of OnOptionsData of the mbtQuotes interface - this NEEDS to be defined, even if you dont need options data
    Private Sub OnOptionsData(ByRef pRec As OPTIONSRECORD) Implements IMbtQuotesNotify.OnOptionsData
    End Sub

    'Implementation of OnTSData of the mbtQuotes interface - this NEEDS to be defined, even if you dont need TS data
    Private Sub OnTSData(ByRef pRec As TSRECORD) Implements IMbtQuotesNotify.OnTSData
    End Sub



    'These subs are used to submit orders

    'this sub submits a marketorder
    Private Sub SubmitMarketOrder(ByVal BuySell As Integer, ByVal pair As String, ByVal price As Double, ByVal quantity As Integer)
        Dim bSubmit As Boolean
        Dim sRetMsg As String
        Dim mlExpMonth As Long
        Dim mlExpYear As Long
        Dim dStrikePrice As Double
        Dim oDte As Date
        Dim route As String

        mlExpMonth = 0
        mlExpYear = 0
        dStrikePrice = 0
        route = "MBTX"
        sRetMsg = ""

        bSubmit = oOrders.Submit(BuySell, quantity, pair, price, 0, VALUE_GTC, VALUE_AGENCY, VALUE_MARKET, VALUE_NORMAL, 0, oAct, route, "", 0, 0, oDte, oDte, mlExpMonth, mlExpYear, 0, 0, 0, sRetMsg)
        PutLog("Submit Order returned: " & bSubmit & " sRetMsg: " & sRetMsg)
    End Sub

    'this sub submits a limitorder
    Private Sub SubmitLimitOrder(ByVal BuySell As Integer, ByVal pair As String, ByVal price As Double, ByVal quantity As Integer)
        Dim bSubmit As Boolean
        Dim sRetMsg As String
        Dim mlExpMonth As Long
        Dim mlExpYear As Long
        Dim dStrikePrice As Double
        Dim oDte As Date
        Dim route As String

        mlExpMonth = 0
        mlExpYear = 0
        dStrikePrice = 0
        route = "MBTX"
        sRetMsg = ""

        bSubmit = oOrders.Submit(BuySell, quantity, pair, price, 0, VALUE_GTC, VALUE_AGENCY, VALUE_LIMIT, VALUE_NORMAL, 0, oAct, route, "", 0, 0, oDte, oDte, mlExpMonth, mlExpYear, 0, 0, 0, sRetMsg)
        PutLog("Submit Order returned: " & bSubmit & " sRetMsg: " & sRetMsg)
    End Sub

    'stopmarketorder
    Private Sub SubmitStopMarketOrder(ByVal BuySell As Integer, ByVal pair As String, ByVal stopprice As Double, ByVal quantity As Integer)
        Dim bSubmit As Boolean
        Dim sRetMsg As String
        Dim mlExpMonth As Long
        Dim mlExpYear As Long
        Dim dStrikePrice As Double
        Dim oDte As Date
        Dim route As String

        mlExpMonth = 0
        mlExpYear = 0
        dStrikePrice = 0
        route = "MBTX"
        sRetMsg = ""

        bSubmit = oOrders.Submit(BuySell, quantity, pair, mdBid, stopprice, VALUE_GTC, VALUE_AGENCY, VALUE_STOP_MARKET, VALUE_NORMAL, 0, oAct, route, "", 0, 0, oDte, oDte, mlExpMonth, mlExpYear, 0, 0, 0, sRetMsg)
        PutLog("Submit Order returned: " & bSubmit & " sRetMsg: " & sRetMsg)
    End Sub

    'limitstopmarketorder
    Private Sub SubmitLimitStopMarketOrder(ByVal BuySell As Integer, ByVal pair As String, ByVal price As Double, ByVal quantity As Integer)
        Dim bSubmit As Boolean
        Dim sRetMsg As String
        Dim mlExpMonth As Long
        Dim mlExpYear As Long
        Dim dStrikePrice As Double
        Dim oDte As Date
        Dim route As String

        mlExpMonth = 0
        mlExpYear = 0
        dStrikePrice = 0
        route = "MBTX"
        sRetMsg = ""

        bSubmit = oOrders.Submit(BuySell, quantity, pair, price, price, VALUE_GTC, VALUE_AGENCY, VALUE_LIMIT_STOPMKT, VALUE_NORMAL, 0, oAct, route, "", 0, 0, oDte, oDte, mlExpMonth, mlExpYear, 0, 0, 0, sRetMsg)
        PutLog("Submit Order returned: " & bSubmit & " sRetMsg: " & sRetMsg)
    End Sub




    'Various other subs

    'sub to write stuff in my log
    Private Sub PutLog(ByVal text As String)
        rtbLog.Text = CStr(Now) & "  " & text & vbNewLine & rtbLog.Text
    End Sub

    'buy button clicked
    Private Sub cmdBuy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAsk.Click
        'lets see how many millisecond it takes until we have an execution
        stopwatch.Start()

        'YEN has only 2 digits after comma
        Dim divisor As Integer
        divisor = IIf(aPair = "USD/JPY", 100, 10000)

        'submit limit or stop order
        If cmbType.SelectedIndex = 0 Then
            SubmitMarketOrder(VALUE_BUY, aPair, mdAsk, nudQuantity.Value * 10000)
        ElseIf cmbType.SelectedIndex = 1 Then
            SubmitLimitOrder(VALUE_BUY, aPair, mdAsk, nudQuantity.Value * 10000)
        End If

        'submit stoploss if activated
        If cbSL.Checked Then
            SubmitStopMarketOrder(VALUE_SELL, aPair, mdAsk - nudSL.Value / divisor, nudQuantity.Value * 10000)
        End If

        'submit targetorder if activated
        If cbTP.Checked Then
            SubmitLimitStopMarketOrder(VALUE_SELL, aPair, mdAsk + nudTP.Value / divisor, nudQuantity.Value * 10000)
        End If
    End Sub

    'Sell button clicked, same as above
    Private Sub cmdSell_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdBid.Click
        stopwatch.Start()
        Dim divisor As Integer
        divisor = IIf(aPair = "USD/JPY", 100, 10000)
        If cmbType.SelectedIndex = 0 Then
            SubmitMarketOrder(VALUE_SELL, aPair, mdBid, nudQuantity.Value * 10000)
        ElseIf cmbType.SelectedIndex = 1 Then
            SubmitLimitOrder(VALUE_SELL, aPair, mdBid, nudQuantity.Value * 10000)
        End If
        If cbSL.Checked Then
            SubmitStopMarketOrder(VALUE_BUY, aPair, mdBid + nudSL.Value / divisor, nudQuantity.Value * 10000)
        End If
        If cbTP.Checked Then
            SubmitLimitStopMarketOrder(VALUE_BUY, aPair, mdBid - nudTP.Value / divisor, nudQuantity.Value * 10000)
        End If
    End Sub

    'this event fires when the user selects another pair from the pull-down menu
    Private Sub cmbPair_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbPair.SelectedIndexChanged
        If bQuotesIsConnected Then
            Select Case cmbPair.SelectedIndex
                Case 0
                    aPair = "EUR/USD"
                    oQuotes.UnadviseAll(Me)
                    oQuotes.AdviseSymbol(Me, aPair, 1)
                    PutLog("Advised " & aPair)
                Case 1
                    aPair = "GBP/USD"
                    oQuotes.UnadviseAll(Me)
                    oQuotes.AdviseSymbol(Me, aPair, 1)
                    PutLog("Advised " & aPair)
                Case 2
                    aPair = "USD/JPY"
                    oQuotes.UnadviseAll(Me)
                    oQuotes.AdviseSymbol(Me, aPair, 1)
                    PutLog("Advised " & aPair)
                Case 3
                    aPair = "USD/CHF"
                    oQuotes.UnadviseAll(Me)
                    oQuotes.AdviseSymbol(Me, aPair, 1)
                    PutLog("Advised " & aPair)
            End Select
        End If
    End Sub


    'these subs display some OCX-controls in seperate forms, see code/designer in the other forms
    Private Sub PositionsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PositionsToolStripMenuItem.Click
        Positions.Show()
    End Sub

    Private Sub OrdersToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OrdersToolStripMenuItem.Click
        Orders.Show()
    End Sub

    Private Sub OrderbookToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OrderbookToolStripMenuItem.Click
        Orderbook.Show()
    End Sub

    Private Sub BalancesToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BalancesToolStripMenuItem.Click
        Balances.Show()
    End Sub

    Private Sub LoginToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LoginToolStripMenuItem.Click
        If Not bOrderClientIsConnected Then
            bOrderClientIsConnected = oComMgr.DoLogin(9, username, password, "")
            PutLog("logging in")
        Else
            PutLog("Already logged in")
        End If
    End Sub

    Private Sub Level2ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Level2ToolStripMenuItem.Click
        Level2.Show()
    End Sub

    Private Sub OPOToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OPOToolStripMenuItem.Click
        OPO.Show()
    End Sub

    Private Sub TimesToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TimesToolStripMenuItem.Click
        Times.Show()
    End Sub
End Class
